#!/usr/bin/php
<?php
/**
 * Generate Alerts
 * The conf.local file allow to set the definitions.
 *
 * <?php
 * $HTMLBASE="/muningraphjs-dev";
 * $alerters = [
 *   "xmpp://muningraph%40xmpp.fournier38.fr:jLDoA@xmpp.fournier38.fr/dominique%40fournier38.fr",
 *   "display://local",
 * ];
 * $disabledAlerts = [
 *   "UNKNOWN : site1 backupfournier.fournier38.fr smart_sda",
 *   "UNKNOWN : site1 proxmox.fournier38.fr smart_sdb",
 *   //"HostDOWN : site2 proxmox2.fournier38.fr",
 * ];
 *
 * Then set the /etc/cron.d/muningraph file with the content :
 *     * * * * * root /path/to/file/cron.php
 */

$alerters = [];
$disabledAlerts = [];
$DATAPATH="/var/lib/munin";
$HTMLBASE="/muningraphjs";
$RRDCACHEADDRESS="unix:/var/run/rrdcached.sock";

@include("conf.local");

spl_autoload_register();
spl_autoload_register(function ($class){
    require_once __DIR__ . "/" . str_replace("\\", "/", $class) . ".php";
});
$munin = new \Munin ();
$munin->datapath ($DATAPATH);
$munin->rrdcacheAddress ($RRDCACHEADDRESS);
$lastHostUpdate = $munin->lastHostUpdate ();
$unknowns = $munin->getUnknown ();
$warnings = $munin->getWarning ();
$criticals = $munin->getCritical ();

$objectsUnknown = [];
foreach($unknowns as $key => $val)
{
  [$site, $host, $module, $internal] = explode ("/", $key);
  $objectsUnknown["$site/$host/$module"][] = $internal;
}
$objectsWarnings = [];
foreach($warnings as $key => $val)
{
  [$site, $host, $module, $internal] = explode ("/", $key);
  $objectsWarnings["$site/$host/$module"][] = $internal;
}
$objectsCriticals = [];
foreach($criticals as $key => $val)
{
  [$site, $host, $module, $internal] = explode ("/", $key);
  $objectsCriticals["$site/$host/$module"][] = $internal;
}
$hostsExpired = [];
foreach ($lastHostUpdate as $key => $val)
{
  if (time() - $val <= 600)
    continue;
  [$site, $host] = explode ("/", $key);
  $hostsExpired["$site/$host"] = $val;
}

$msg = "";
foreach ($hostsExpired as $key => $val)
{
  // To disable the alert, do not put the (since X minutes) in the conf file
  if (in_array ("HostDOWN : " . str_replace ("/", " ", $key), $disabledAlerts))
    continue;
  $msg .= "HostDOWN : " . str_replace ("/", " ", $key) .
          " (since " . intval((time() - $val) / 60) .
          " minutes)\n";
}
foreach ($objectsCriticals as $key => $val)
{
  if (in_array ("CRITICAL : " . str_replace ("/", " ", $key), $disabledAlerts))
    continue;
  $msg .= "CRITICAL : " . str_replace ("/", " ", $key) ."\n";
}
foreach ($objectsWarnings as $key => $val)
{
  if (in_array ("WARNING : " . str_replace ("/", " ", $key), $disabledAlerts))
    continue;
  $msg .= "WARNING : " . str_replace ("/", " ", $key) ."\n";
}
foreach ($objectsUnknown as $key => $val)
{
  if (in_array ("UNKNOWN : " . str_replace ("/", " ", $key), $disabledAlerts))
    continue;
  $msg .= "UNKNOWN : " . str_replace ("/", " ", $key) ."\n";
}
if ($msg != "")
{
  foreach ($alerters as $alerter)
  {
    $url = parse_url($alerter);
    if ($url === false)
      die ("Can not use the alerter '$alerter'\n");
    switch($url["scheme"])
    {
    case "display":
      echo "\n'".gethostname(). "' has detected the following problems :\n\n\n";
      echo $msg;
      echo "\n\nYou can disable the alert (if needed) in '". __DIR__ . "/conf.local' file\n";
      break;
    case "xmpp":
      $url["port"] = $url["port"] ?? 5222;
      if (! isset($url["host"]) || ! isset($url["user"]) || ! isset($url["pass"]) || ! isset($url["path"]))
        die ("Invalid configuration for alerter '$alerter' : missing parts\n");
      $xmpp = new \Domframework\Xmppclient ();
      $xmpp->connect($url["host"], $url["port"], urldecode($url["user"]), urldecode($url["pass"]));
      $xmpp->sendMessagePrivate(urldecode(ltrim($url["path"], "/")),
        $msg.
        "You can disable the alert (if needed) in '". __DIR__ . "/conf.local' file\n",
        "Munin Alert");
      $xmpp->disconnect();
      break;
    default:
      die ("Unknown alerter '$alerter'\n");
    }
  }
}
