<?php
/**
  * Munin Graph JS
  * @author Dominique Fournier <dominique@fournier38.fr>
  * Licence BSD
  */

/////////////////////////////
////    CONFIGURATION    ////
/////////////////////////////
//$DATAPATH="data";
//$HTMLBASE="/ProgSVN/muningraphjs";
//$RRDCACHEaDDRESS="";
$DATAPATH="/var/lib/munin";
$HTMLBASE="/muningraphjs";
$RRDCACHEADDRESS="unix:/var/run/rrdcached.sock";
$COLORS = [
  '#1f77b4',  // muted blue
  '#ff7f0e',  // safety orange
  '#2ca02c',  // cooked asparagus green
  '#d62728',  // brick red
  '#9467bd',  // muted purple
  '#8c564b',  // chestnut brown
  '#e377c2',  // raspberry yogurt pink
  '#7f7f7f',  // middle gray
  '#bcbd22',  // curry yellow-green
  '#17becf'   // blue-teal
];

@include ("conf.local");

///////////////////////////////////
////    GET DATA FROM MUNIN    ////
///////////////////////////////////
spl_autoload_register();
$munin = new \Munin ();
$munin->datapath ($DATAPATH);
$munin->rrdcacheAddress ($RRDCACHEADDRESS);
$datafileContent = $munin->readDataFile ();
$groupsHosts = $munin->groupsHosts ();
$groupsHostsModules = $munin->groupsHostsModules ();
$groupsHostsModulesInternal = $munin->groupsHostsModulesInternal ();
$lastHostUpdate = $munin->lastHostUpdate ();

$GLOBALS["timeRRD"] = 0;
$start = microtime (true);
/////////////////////////////////////////////////////
////    GET THE GROUP/HOST/MODULES TO DISPLAY    ////
/////////////////////////////////////////////////////
if (! isset ($_SERVER["QUERY_STRING"]))
  $_SERVER["QUERY_STRING"] = "uri=";
$url = substr ($_SERVER["QUERY_STRING"], 5);
if (substr ($url, -1) === "/")
  $url = substr ($url, 0, -1);
if (substr ($url, 0, 4) === "api/")
{
  // In API, we get the values of the URL like :
  //     /api/group/host/module/timeperiod
  @list ($api, $group, $host, $module, $timeperiod) = explode ("/", $url);
  $name = "$group/$host/$module";
  $internals = array ();
  foreach ($groupsHostsModulesInternal as $ghmi)
  {
    if (substr ($ghmi, 0, strlen ($name) + 1) !== "$name/")
      continue;
    $internals[] = substr ($ghmi, strlen ($name) + 1);
  }
  if (empty ($internals))
  {
    http_response_code (404);
    die ("URL '$name' doesn't exists");
  }
  $titles = "time";
  $values = array ();
  foreach ($internals as $internal)
  {
    try
    {
      $values[$internal] = $munin->getValues ($name, $internal, $timeperiod);
      $titles .= ",$internal";
    }
    catch (\Exception $e)
    {

    }
  }
  // Convert to array for CSV
  $res = array ();
  foreach ($values as $internal => $vals)
  {
    foreach ($vals as $key => $val)
    {
      if (! key_exists ($key, $res))
        $res[$key] = "$key";
      $res[$key] .= ",$val";
    }
  }
  header ("Content-Type: text/csv");
  echo "$titles\n";
  echo implode ("\n", $res);
  exit;
}
elseif (! in_array ($url, $groupsHosts) &&
        ! in_array ($url, $groupsHostsModules))
  $url = "";

////////////////////////////////
////    DISPLAY THE PAGE    ////
////////////////////////////////
echo "<!DOCTYPE html>\n";
echo "<html>\n";
echo "  <head>\n";
echo "    <meta http-equiv='Content-Type' content='text/html; charset=utf-8' ".
     "/>\n";
echo "    <meta http-equiv='X-UA-Compatible' content='IE=edge'>\n";
echo "    <meta name='viewport' content='width=device-width, initial-scale=1, ".
     "maximum-scale=1, user-scalable=no'>\n";
if ($url === "")
  echo "    <title>Munin</title>\n";
else
  echo "    <title>Munin - ".str_replace ("/", " :: ", $url)."</title>\n";
echo "    <style>\n";
// {{{
echo "
body {
  font-family: Helvetica, Arial, sans-serif;
  display: grid;
  grid-template-columns: 1fr;
  grid-template-areas:
    'sidebar'
    'content'
    'footer'
}
@media screen and (min-width: 1024px)
{
  body
  {
    grid-template-columns: auto;
    grid-template-areas:
      'sidebar content'
      'footer footer'
  }
}
h1 {
  text-align: center;
}
nav, article, div {
  vertical-align: top;
}
nav {
  grid-area: sidebar;
  padding-right: 30px;
  transform: scale(1);
  transition: 0.5s all ease;
}
article {
  grid-area: content;
}
footer {
  grid-area: footer;
}
.logo {
  text-align: center;
}
a {
  text-decoration: none;
  color: #459;
}
a:hover {
  color: #229;
}
ul {
  margin-top: 0;
}
li {
  list-style-type: none;
  margin-left: -30px;
}
.graph {
  display: inline-block;
  margin: 0 auto;
  border: 1px solid #ddd;
  height: 400px;
  width: 800px;
  background-color: #eee;
}
.data {
  display: inline-block;
  margin: 0 auto;
  border: 1px solid #ddd;
  height: 400px;
  width: 800px;
  background-color: #fff;
  overflow: auto;
}
.data.clickable {
  cursor: pointer;
}
.headerModulesTop {
  text-align: center;
}

table {
  margin-left: auto;
  margin-right: auto;
}
tr {
}
th {
  background-color: #eee;
}
td {
  margin-right: 3px;
  border-bottom: 1px solid #eee;
}

.host {
  color: #555;
  font-weight: bold;
}
li {
  padding-bottom: 2px;
}
.unknown {
  border-color: #ccc;
}
.warning {
  border-color: orange;
}
.critical {
  border-color: #ffa0ab;
}
h2.unknown a {
  color: #777;
}
h2.warning a {
  color: orange;
}
h2.critical a {
  color: #ffa0ab;
}
tr.unknown,
li.unknown,
.headerModules .unknown,
.headerModulesTop .unknown {
  background-color: #ccc;
}
tr.warning,
li.warning,
.headerModules .warning,
.headerModulesTop .warning {
  background-color: orange;
}
tr.critical,
li.critical,
.headerModules .critical,
.headerModulesTop .critical {
  background-color: #ffa0ab;
}
\n";
// }}}
echo "    </style>\n";
echo "    <meta http-equiv='refresh' content='300'>\n";
echo "    <script src='$HTMLBASE/public/plotly-1.51.0.min.js'></script>\n";
echo "  </head>\n";
echo "  <body>\n";
$unknowns = $munin->getUnknown ();
$warnings = $munin->getWarning ();
$criticals = $munin->getCritical ();
// The navigation bar on left
echo "    <nav>\n";
echo "      <p class='logo'><a href='$HTMLBASE/'>".
     "<img src='$HTMLBASE/public/logo-h.png' alt='Munin Logo'/></a></p>\n";
$lastgroup = "";
foreach ($groupsHosts as $name)
{
  list ($group, $host) = explode ("/", $name);
  if ($group !== $lastgroup)
  {
    if ($lastgroup !== "")
      echo "      </ul>\n";
    echo "$group::\n";
    echo "      <ul>\n";
    $lastgroup = $group;
  }
  $state = "";
  if ($url !== "")
    $state = getState ("$group/$host", $criticals, $warnings, $unknowns);
  $hoststate = "";
  if  ($lastHostUpdate["$group/$host"] < time () - 600)
    $state = "critical";
  echo "        <li class='headerModules $state'>";
  echo "<a href='$HTMLBASE/$group/$host/' class='host $hoststate'>";
  echo $host;
  echo "</a>";
  if ($url === "")
  {
    // Display the list of module if no module is displayed
    echo "<br/> ";
    modulesListByHost ($groupsHostsModules, $group, $host,
      $criticals, $warnings, $unknowns, $HTMLBASE);
    echo " ";
  }
  echo "</li>\n";
}
echo "      </ul>\n";
echo "    </nav>\n";
// The main page on right
echo "    <article>\n";
if (substr_count ($url, "/") === 1 || substr_count ($url, "/") === 2)
{
  echo "      <h1>Display ".str_replace ("/", " :: ", $url)."</h1>\n";
  @list ($group, $host, $module) = explode ("/", $url);
  echo "      <div class='headerModulesTop'>\n";
  modulesListByHost ($groupsHostsModules, $group, $host,
    $criticals, $warnings, $unknowns, $HTMLBASE);
  echo "      </div>\n";
  // Display all the available modules for the provided host or only one if
  // it is forced on URL
  $toDisplay = array ();
  foreach ($groupsHostsModules as $name)
  {
    if (substr ($name, 0, strlen ($url)) !== $url)
      continue;
    $toDisplay[] = $name;
  }
  $lastValues = array ();
  foreach ($toDisplay as $name)
  {
    list ($group, $host, $module) = explode ("/", $name);
    $vals = $datafileContent[$name];
    $state = getState ($name, $criticals, $warnings, $unknowns);
    echo "      <h2 id='$module' class='$state'>".
         "<a href='$HTMLBASE/$group/$host/$module'>$module</a></h2>\n";
    if (count ($toDisplay) === 1)
    {
      // Display 4 graphs : daily, weekly monthly and yearly
      foreach (["day", "week", "month", "year"] as $timeperiod)
      {
        $values = graph ("$name", $vals, $munin, $state, $timeperiod);
        if ($timeperiod === "day")
          $lastValues = $lastValues + $values;
      }
      tablevalues ($name, $vals, $munin, $lastValues,
                   $criticals, $warnings, $unknowns);
    }
    else
    {
      // Display only the daily graph as there is some other graphs to display
      echo "<a href='$HTMLBASE/$group/$host/$module'>";
      $values = graph ("$name", $vals, $munin, $state, "day");
      echo "</a>";
      $lastValues = $lastValues + $values;
      tablevalues ($name, $vals, $munin, $lastValues,
                   $criticals, $warnings, $unknowns,
                   "$HTMLBASE/$group/$host/$module");
    }
    //ob_flush ();
    //flush ();
  }
}
echo "    </article>\n";
echo "    <footer>\n";
echo "Last update ".date ("d/m/Y H:i");
echo "    </footer>\n";
if ($url !== "")
{
  // The autoscroll must not be enabled on main page
// {{{
echo "<script>
// Manage the menu scroll when the user scroll the main window
var scrolling = false;
window.addEventListener ('scroll', function() {
  scrolling = true;
});
function scroll ()
{
  offset =  window.pageYOffset + 'px';
  if (window.innerWidth < 1024)
    offset = 0;
  document.querySelector ('nav').style.paddingTop = offset
}
setInterval( function() {
  if ( scrolling ) {
    scrolling = false;
    scroll ();
  }
}, 1750 );
window.addEventListener ('resize', function () {
  // Wait the end of modification size before applying the scroll
  setTimeout (function () {
    scroll ();
  }, 50);
});
scroll ();
</script>\n";
// }}}
}
echo "<script>\n";
$val = <<< 'EOT'
function humanSizeJS ($value, $decimals = 2, $power = 1000, $unit = "B")
  {
    if ($value === "")
      return "";
    $value = parseFloat($value);
    if (typeof ($value) !== "number")
      throw ("convert::humanSizeJS value not numerical : "+
        typeof $value);
    if (! Number.isInteger ($decimals))
      throw ("convert::humanSizeJS decimal not integer : "+
        typeof $decimals);
    if ($decimals < 0)
      throw ("convert::humanSizeJS decimal value negative", 500);
    if ($power !== 1000 && $power !== 1024)
      throw ("convert::humanSizeJS power value !== 1000 and 1024"+
      " : " + $power, 500);
    $size = {
      '-8' : "y", // yocto
      '-7' : "z", // zepto
      '-6' : "a", // atto
      '-5' : "f", // femto
      '-4' : "p", // pico
      '-3' : "n", // nano
      '-2' : "u", // micro
      '-1' : "m", // milli
      '0' : " ",
      '1' : "k", // kilo
      '2' : "M", // mega
      '3' : "G", // giga
      '4' : "T", // tera
      '5' : "P", // peta
      '6' : "E", // exa
      '7' : "Z", // zetta
      '8' : "Y"};// yotta
    for ($factor = 8 ; $factor > -8 ; $factor--)
    {
      if (Math.abs ($value) >= Math.pow ($power, $factor))
        break;
    }
    if ($value == 0)
    {
      $display = 0;
      $factor = 0;
    }
    else if ($factor > 0)
      $display = $value / Math.pow ($power, $factor);
    else if ($factor < 0)
      $display = $value * Math.pow ($power, -1 * $factor);
    else
      $display = $value;

    return $display.toFixed($decimals)+$size[$factor]+$unit;
  }
EOT;
echo $val;
echo "</script>\n";
echo "  </body>\n";
echo "</html>\n";

/** Display the table with the module values
  * @param string $name The name of group/host/module
  * @param array $vals The vals get from $datafileContent[$name]
  * @param object $munin The munin object
  * @param array $lastValues The last value for each module name
  * @param array $criticals The internal in critical
  * @param array $warnings The internal in warning
  * @param array $unknowns The internal in unknown
  * @param string|null $clickable The URL to go when clicking on div
  */
function tablevalues ($name, $vals, $munin, $lastValues,
  $criticals, $warnings, $unknowns, $clickable = null)
// {{{
{
  $state = getState ($name, $criticals, $warnings, $unknowns);
  echo "<div class='data $state";
  if ($clickable)
    echo "clickable' onClick=\"document.location.href='$clickable'\"";
  else
    echo "'";
  echo ">\n";
  if (key_exists ("graph_info", $vals))
    echo "  <p style='text-align:center'>".$vals["graph_info"]."</p>\n";
  elseif (key_exists ("graph_title", $vals))
    echo "  <p style='text-align:center'>".$vals["graph_title"]."</p>\n";
  echo "  <table>\n";
  echo "    <tr><th>Label</th><th>Internal Name</th><th>Actual Value</th>".
       "<th>Warning</th><th>Critical</th><th>Message</th></tr>\n";
  $internals = array ();
  // Get the rrd names to graph with their order
  if (key_exists ("graph_order", $vals))
  {
    $internals = explode (" ", $vals["graph_order"]);
  }
  else
  {
    foreach ($vals as $key => $val)
    {
      if (is_array ($val))
        $internals[] = $key;
    }
  }
  $internals = array_values (array_unique ($internals));
  // Check if the values are stacked
  foreach ($internals as $internal)
  {
    if (! key_exists ($internal, $vals))
      continue;
    $val = $vals[$internal];
    $label = isset ($val["label"]) ? $val["label"]: $internal;
    $warning = isset ($val["warning"]) ? $val["warning"]: "";
    $critical = isset ($val["critical"]) ? $val["critical"] : "";
    $message = "";
    echo "    <tr";
    $state = getState ("$name/$internal", $criticals, $warnings, $unknowns);
    echo " class='$state'";
    $message = getMessage ("$name/$internal", $criticals, $warnings, $unknowns);
    echo "><td>$label</td><td>$internal</td>".
         "<td style='text-align:right' id='${name}_${internal}_val'></td>".
         "<td style='text-align:center'>$warning</td>".
         "<td style='text-align:center'>$critical</td>".
         "<td style='text-align:center'>$message</td></tr>\n";
  }
  echo "  </table>\n";
//echo "<pre>";
//print_r ($datafileContent[$name]);
//echo "</pre>";
  echo "</div>\n";
}
// }}}

/** Graph the values for the provided name module
  * The graph must have already be set on graph_$module div id
  * @param string $name The name of group/host/module
  * @param array $vals The vals get from $datafileContent[$name]
  * @param object $munin The munin object
  * @param string $state The state of the current plugin to color in red in
  * case of critical state
  * @param string $timeperiod in "day", "week", "month", "year"
  * @return the last value get from RRD
  */
function graph ($name, $vals, $munin, $state, $timeperiod)
// {{{
{
  global $COLORS, $HTMLBASE;
  list ($group, $host, $module) = explode ("/", $name);
  echo "<div id='graph_${module}_${timeperiod}' class='graph $state'></div>\n";
  echo "<script>\n";
  $internals = array ();
  // Get the rrd names to graph with their order
  if (key_exists ("graph_order", $vals))
  {
    $internals = explode (" ", $vals["graph_order"]);
  }
  else
  {
    foreach ($vals as $key => $val)
    {
      if (is_array ($val))
        $internals[] = $key;
    }
  }
  $internals = array_values (array_unique ($internals));
  // Check if the values are stacked
  $stackgroupReq = "";
  foreach ($vals as $key => $val)
  {
    if (! is_array ($val))
      continue;
    if (key_exists ("draw", $val) && $val["draw"] === "STACK")
      $stackgroupReq = "stack";
  }
  $colorUsed = -1;
  $lastValues = array ();
  $period = "second";
  $vlabel = "";
  $title = "";
  $logarithmic = "";
  $min = "";
  if (key_exists ("graph_period", $vals))
    $period = $vals["graph_period"];
  if (key_exists ("graph_vlabel", $vals))
    $vlabel = str_replace ("\${graph_period}", $period,
              $vals["graph_vlabel"]);
  if (key_exists ("graph_title", $vals))
    $title = str_replace ("\${graph_period}", $period,
             $vals["graph_title"]);
  if (key_exists ("graph_args", $vals))
  {
    if (strpos ($vals["graph_args"], "--logarithmic") !==false)
      $logarithmic = "true";
    if (strpos ($vals["graph_args"], "-l 0") !== false)
      $min = 0;
  }

  echo "Plotly.d3.csv (\"$HTMLBASE/api/$group/$host/$module/$timeperiod\",".
       " function(data) {\n";
  echo "var time = []";
  foreach ($internals as $nb => $internal)
  {
    if ($internal === "")
    {
      unset ($internals[$nb]);
      continue;
    }
    echo ", $internal = []";
  }
  echo ";\n";
  // Convert CSV to JS variables
  echo "  for (var i = 0 ; i < data.length ; i++) {\n";
  echo "    var row = data[i];\n";
  echo "    time.push (row['time']);\n";
  foreach ($internals as $nb => $internal)
  {
    echo "    $internal.push (parseFloat(row['$internal']));\n";
  }
  echo "  }\n";

  // Display the value in data
  echo "  var row = data[i - 2];\n";
  foreach ($internals as $nb => $internal)
  {
    echo "  var val = row['$internal'];\n";
    echo "  if (val == '' || val == undefined) val = 'U';\n";
    echo "  else val = humanSizeJS (val, 3, 1000, '');\n";

    echo "  document.getElementById ('${name}_${internal}_val').innerText = ".
         " val;\n";
  }

  foreach ($internals as $nb => $internal)
  {
    $fill = "";
    $stackgroup = "";
    $showlegend = "";
    if (key_exists ("graph", $vals[$internal]) &&
        $vals[$internal]["graph"] === "no")
      $showlegend = "false";
    if (key_exists ("draw", $vals[$internal]))
    {
      if ($vals[$internal]["draw"] === "AREA" &&
          $stackgroupReq === "")
        $fill =  "tozeroy";
      elseif ($vals[$internal]["draw"] === "AREA" &&
          $stackgroupReq !== "")
        $stackgroup = "stack";
      elseif ($vals[$internal]["draw"] === "AREASTACK")
        $fill = "tonexty";
      elseif ($vals[$internal]["draw"] === "STACK")
        $stackgroup = "stack";
    }
    if (!key_exists ("negative", $vals[$internal]) &&
        $showlegend !== "false")
      $colorUsed++;
    echo "  var trace$nb = {\n";
    echo "    x: time,\n";
    echo "    y: $internal,\n";
    echo "    connectgaps: false,\n";
    if (key_exists ("label", $vals[$internal]))
      echo "    name: '".$vals[$internal]["label"]."',\n";
    else
      echo "    name: '$internal',\n";
    if ($fill !== "")
      echo "    fill: '$fill',\n";
    if ($stackgroup !== "")
      echo "    stackgroup: '$stackgroup',\n";
    if ($showlegend === "false")
      echo "    showlegend: false,\n";
    if ($stackgroup === "")
    {
      echo "    mode: 'lines',\n";
      echo "    line: {\n";
      echo "      color: '".$COLORS[$colorUsed%count($COLORS)]."',\n";
      //echo "      width: 3,\n";
      echo "    },\n";
    }
    echo "  };\n";
  }
  // Send the main part
  echo "  var data  = [ trace".implode (", trace", array_keys ($internals)).
       " ];\n";
  echo "  var layout = {\n";
  if ($title !== "")
    echo "    title: '$title - by $timeperiod',\n";
  echo "    xaxis: {\n";
  echo "      exponentformat: 'SI',\n";
  echo "      tickmode: 'auto',\n";
  echo "      nticks: 34,\n";
  echo "    },\n";
  echo "    yaxis: {\n";
  echo "      exponentformat: 'SI',\n";
  echo "      tickmode: 'auto',\n";
  echo "      nticks: 20,\n";
  if ($vlabel !== "")
    echo "      title: '$vlabel',\n";
  if ($logarithmic !== "")
  {
    echo "      type: 'log',\n";
    echo "      autorange: true,\n";
  }
  if ($min !== "")
  {
    echo "      rangemode: 'tozero',\n";
    echo "      autorange: true,\n";
  }
  echo "    },\n";
  echo "  };\n";
  echo "  Plotly.newPlot('graph_${module}_${timeperiod}', data, layout, ".
       "{responsive: true});\n";
  echo "});\n";
/*




  foreach ($internals as $nb => $internal)
  {
    if ($internal === "")
      continue;
    $fill = "";
    $stackgroup = "";
    $showlegend = "";
$start = microtime (true);
    $values = $munin->getValues ($name, $internal, $timeperiod);
$end = microtime (true) - $start;

$GLOBALS["timeRRD"] += $end;
    // Remove the last always empty value
    array_pop ($values);
    $lastValues["$name/$internal"] = end ($values);

    echo "// COLORUSED => $colorUsed\n";
    // Send the values
    echo "var trace$nb = {\n";
    echo "  x: [ '".implode ("', '", array_keys ($values))."'],\n";
    echo "  y: [ ".implode (", ", $values)." ],\n";
    echo "  connectgaps: false,\n";
    if (key_exists ("label", $vals[$internal]))
      echo "  name: '".$vals[$internal]["label"]."',\n";
    else
      echo "  name: '$internal',\n";
    if ($fill !== "")
      echo "  fill: '$fill',\n";
    if ($stackgroup !== "")
      echo "  stackgroup: '$stackgroup',\n";
    if ($showlegend === "false")
      echo "  showlegend: false,\n";
    if ($stackgroup === "")
    {
      echo "  mode: 'lines',\n";
      echo "  line: {\n";
      echo "    color: '".$COLORS[$colorUsed%count($COLORS)]."',\n";
      //echo "    width: 3,\n";
      echo "  },\n";
    }
    echo "};\n";
  }

  echo "Plotly.newPlot('graph_${module}_${timeperiod}', data, layout, ".
       "{responsive: true});\n";
*/
  echo "</script>\n";
  return $lastValues;
}
// }}}

/** Display the list of modules enabled for a the provided host.
  */
function modulesListByHost ($groupsHostsModules, $group, $host,
  $criticals, $warnings, $unknowns, $HTMLBASE)
// {{{
{
  foreach ($groupsHostsModules as $full)
  {
    if (substr ($full, 0, strlen ("$group/$host")) !== "$group/$host")
      continue;
    $module = substr ($full, strlen ("$group/$host") + 1);
    echo " &nbsp;&nbsp; <a href='$HTMLBASE/$group/$host/#$module'";
    $name = "$group/$host/$module";
    $state = getState ($name, $criticals, $warnings, $unknowns);
    echo " class='$state'>$module</a>\n";
  }
  echo "\n";
}
// }}}

/** This function return the state (critical, warning, unknown, "") of the
  * provided $name or $nameInternal
  * @param string $nameInternal the object to examine
  * @param array $criticals The objects in critical state
  * @param array $warnings The objects in warning state
  * @param array $unknowns The objects in unknown state
  */
function getState ($nameInternal, $criticals, $warnings, $unknowns)
// {{{
{
  @list ($group, $host, $module, $internal) = explode ("/", $nameInternal);
  $state = "";
  if (key_exists ($nameInternal, $criticals))
    $state = "critical";
  elseif (key_exists ($nameInternal, $warnings))
    $state = "warning";
  elseif (key_exists ($nameInternal, $unknowns))
    $state = "unknown";

  foreach ($criticals as $key => $message)
  {
    if ($state !== "")
      continue;
    if (substr ($key, 0, strlen ($nameInternal) + 1) === $nameInternal . "/")
      $state = "critical";
  }
  foreach ($warnings as $key => $message)
  {
    if ($state !== "")
      continue;
    if (substr ($key, 0, strlen ($nameInternal) + 1) === $nameInternal . "/")
      $state = "warning";
  }
  foreach ($unknowns as $key => $message)
  {
    if ($state !== "")
      continue;
    if (substr ($key, 0, strlen ($nameInternal) + 1) === $nameInternal . "/")
      $state = "unknown";
  }
  return $state;
}
// }}}

/** This function return the state message in case of error
  */
function getMessage ($nameInternal, $criticals, $warnings, $unknowns)
// {{{
{
  if (key_exists ($nameInternal, $unknowns))
    return $unknowns[$nameInternal];
  elseif (key_exists ($nameInternal, $warnings))
    return $warnings[$nameInternal];
  elseif (key_exists ($nameInternal, $criticals))
    return $criticals[$nameInternal];
  return "";
}
// }}}
